(*
 * Interpreter for F - Elaboration
 *
 * 2000/02/02 Andreas Rossberg
 *)


structure Elab :> Elab =
  struct
    open Syntax
    open Env

    datatype error = IdUnbound of id
		   | IfConditionNotBool
		   | IfBranchesMismatch
		   | AppNoFunction
		   | AppArgumentMismatch

    exception Error of error


    fun elab T (Con c)		= Type.constant c
      | elab T (Id x)		= (lookup(x,T) handle Unbound x =>
				      raise Error(IdUnbound x))
      | elab T (If(e1,e2,e3))	= let
				      val t1 = elab T e1
				      val t2 = elab T e2
				      val t3 = elab T e3
				  in
				      if t1 <> Bool then
					  raise Error IfConditionNotBool
				      else if t2 <> t3 then
					  raise Error IfBranchesMismatch
				      else
					  t2
				  end
      | elab T (App(e1,e2))	= let
				      val t1 = elab T e1
				      val t2 = elab T e2
				  in
				      case t1
					of Arrow(t11,t12) =>
					    if t11 <> t2 then
						raise Error AppArgumentMismatch
				 	    else
						t12
					 | _ => raise Error AppNoFunction
				  end
      | elab T (Abs(x,t,e))	= let
				      val t2 = elab (insert(x,t,T)) e
				  in
				      Arrow(t,t2)
				  end
  end
