(*
 * Interpreter for F - Lexer
 *
 * 2000/02/02 Andreas Rossberg
 *)


structure Lexer :> Lexer =
  struct
    open Token

    exception Error

    fun digit c				= Char.ord c - Char.ord #"0"

    fun idOrRes l			= idOrRes'(String.implode(List.rev l))
    and idOrRes' "if"			= IF
      | idOrRes' "then"			= THEN
      | idOrRes' "else"			= ELSE
      | idOrRes' "fn"			= FN
      | idOrRes' "bool"			= BOOL
      | idOrRes' "int"			= INT
      | idOrRes'  s			= ID s


    fun munch ts []			= List.rev ts
      | munch ts (#" " :: cs)		= munch ts cs
      | munch ts (#"\t" :: cs)		= munch ts cs
      | munch ts (#"\n" :: cs)		= munch ts cs
      | munch ts (#"(" :: cs)		= munch (LPAR::ts) cs
      | munch ts (#")" :: cs)		= munch (RPAR::ts) cs
      | munch ts (#":" :: cs)		= munch (COLON::ts) cs
      | munch ts (#"-" :: #">" :: cs)	= munch (ARROW::ts) cs
      | munch ts (#"=" :: #">" :: cs)	= munch (DARROW::ts) cs
      | munch ts (cs as c :: _)		= if Char.isDigit c then con ts cs
					  else if Char.isAlpha c then id ts cs
					  else raise Error

    and id  ts cs			= id' ts [] cs
    and id' ts l   []			= munch (idOrRes l :: ts) []
      | id' ts l (c::cs)		= if Char.isAlphaNum c
					  then id' ts (c::l) cs
					  else munch (idOrRes l :: ts) (c::cs)

    and con  ts cs			= con' ts 0 cs
    and con' ts n   []			= munch (CON n :: ts) []
      | con' ts n (c::cs)		= if Char.isDigit c
					  then con' ts (10*n + digit c) cs
					  else munch (CON n :: ts) (c::cs)


    fun lex s				= munch [] (String.explode s)

  end
